@@ -39,12 +39,12 @@
 
 #include <glib/gi18n.h>
 #include <gtksourceview/gtksourcestyleschememanager.h>
+#include <gtksourceview/gtksourcelanguagemanager.h>
 
 #include "gedit-settings.h"
 #include "gedit-document.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
-#include "gedit-language-manager.h"
 #include "gedit-document-loader.h"
 #include "gedit-document-saver.h"
 #include "gedit-marshal.h"
@@ -878,7 +878,7 @@ guess_language (GeditDocument *doc,
 		if (strcmp (data, "_NORMAL_") != 0)
 		{
 			language = gtk_source_language_manager_get_language (
-						gedit_get_language_manager (),
+						gtk_source_language_manager_get_default (),
 						data);
 		}
 
@@ -902,7 +902,7 @@ guess_language (GeditDocument *doc,
 		}
 
 		language = gtk_source_language_manager_guess_language (
-					gedit_get_language_manager (),
+					gtk_source_language_manager_get_default (),
 					basename,
 					content_type);
 
